@@ -1,8 +1,9 @@
 from .iplugin import IPlugin
+from .directory_manager import DirectoryManager
 from .file_manager import FileManager
 from .module_manager import ModuleManager
 from .plugin_manager import PluginManager
 from .plugin_interface import PluginInterface
 
-__all__ = ["IPlugin", "FileManager", "ModuleManager",
-           "PluginManager", "PluginInterface"]
+__all__ = ["IPlugin", "DirectoryManager", "FileManager",
+           "ModuleManager", "PluginManager", "PluginInterface"]
