@@ -7,6 +7,24 @@ if is_py2:
     import shutil as _shutil
     import warnings as _warnings
     from tempfile import mkdtemp
+    from ConfigParser import ConfigParser
+
+    def read_dict(self, dictionary, source='<dict>'):
+        elements_added = set()
+        for section, keys in dictionary.items():
+            section = str(section)
+            try:
+                self.add_section(section)
+            except:
+                pass
+            elements_added.add(section)
+            for key, value in keys.items():
+                key = self.optionxform(str(key))
+                if value is not None:
+                    value = str(value)
+                elements_added.add((section, key))
+                self.set(section, key, value)
+    ConfigParser.read_dict = read_dict
     FILE_ERROR = OSError
 
     class TemporaryDirectory(object):
